//------------------------------------------------------------------------------
// Copyright (C) 2010 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
package zalewa.delayedproxy.network;

import java.net.DatagramPacket;
import java.util.Date;

/**
 * 
 * A structure containing information about a packet that should be sent only
 * when a defined period of time has passed.
 * 
 */
public class DelayedDatagramPacket
{
	public DatagramPacket packet = null;

	private int delayMs = 0;
	private Date dateIssued = new Date();

	public DelayedDatagramPacket(DatagramPacket packet)
	{
		this.packet = packet;
	}

	public DelayedDatagramPacket(DatagramPacket packet, int delayMs)
	{
		setDelay(delayMs);
		this.packet = packet;
	}

	/**
	 * Checks if packet has already been delayed for delayMs amount of
	 * miliseconds.
	 * 
	 * @return True if packet was delayed enough.
	 */
	public boolean isAfterDelay()
	{
		Date current = new Date();
		long timeDifference = current.getTime() - dateIssued.getTime();

		return timeDifference >= (long) delayMs;
	}

	public int getDelay()
	{
		return delayMs;
	}

	/**
	 * 
	 * @param miliseconds
	 *            Cannot be negative.
	 */
	public void setDelay(int miliseconds)
	{
		if (miliseconds < 0)
		{
			miliseconds = 0;
		}

		delayMs = miliseconds;
	}

}
