package zalewa.delayedproxy.network;

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;

import zalewa.delayedproxy.exceptions.IncorrectPortNumberException;

public class HostInfo
{
	public InetAddress address = null;
	public int port = 1;

	public HostInfo()
	{

	}

	/**
	 * Attempts to create HostInfo instance through
	 * setHostInfoFromStringAddressPort() method.
	 * 
	 * @param addressString
	 * @throws MalformedURLException
	 * @throws UnknownHostException
	 * @throws IncorrectPortNumberException
	 */
	public HostInfo(String addressString) throws MalformedURLException,
			UnknownHostException, IncorrectPortNumberException
	{
		setHostInfoFromStringAddressPort(addressString);
	}

	public HostInfo(String hostName, int port) throws UnknownHostException
	{
		this(InetAddress.getByName(hostName), port);
	}

	public HostInfo(InetAddress address, int port)
	{
		this.address = address;
		this.port = port;
	}

	@Override
	public boolean equals(Object obj)
	{
		if (obj instanceof HostInfo)
		{
			HostInfo other = (HostInfo) obj;

			return this.address.equals(other.address)
					&& this.port == other.port;
		}
		else
		{
			return super.equals(obj);
		}
	}

	public void setAddressFromHostName(String hostName)
			throws UnknownHostException
	{
		this.address = InetAddress.getByName(hostName);
	}

	/**
	 * 
	 * @param addressString
	 *            String in format &lt;host&gt;[:&lt;port&gt;]
	 * @throws MalformedURLException
	 *             Thrown if addressString format was incompatible with
	 *             expected.
	 * @throws UnknownHostException
	 *             Thrown if host name couldn't be resolved.
	 * @throws IncorrectPortNumberException
	 *             Thrown if port string cannot be parsed or port number is
	 *             incorrect.
	 */
	public void setHostInfoFromStringAddressPort(String addressString)
			throws IncorrectPortNumberException, MalformedURLException,
			UnknownHostException
	{
		String[] splitString = addressString.split(":");

		if (splitString.length > 2 || splitString[0].isEmpty())
		{
			throw new MalformedURLException();
		}

		String host = splitString[0];

		if (splitString.length > 1)
		{
			try
			{
				port = Integer.parseInt(splitString[1]);
				validatePort(port);
			}
			catch (Exception exc)
			{
				throw new IncorrectPortNumberException();
			}
		}

		this.address = InetAddress.getByName(host);
	}
	
	public static void validatePort(int port) throws IncorrectPortNumberException
	{
		if (port < 1 || port > 65535)
		{
			throw new IncorrectPortNumberException();
		}
	}
}
