//------------------------------------------------------------------------------
// Copyright (C) 2010 "Zalewa" <zalewapl@gmail.com>
//------------------------------------------------------------------------------
package zalewa.delayedproxy.network;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketTimeoutException;
import java.util.Date;

public class ServerSender extends DatagramBase implements Runnable
{
	public static final long MAX_LAST_RECEIVED_TIME_MS = 30 * 1000;
	
	private boolean bKeepRunning = true;
	private HostInfo clientInfo = null;
	private Date lastReceivedPacketTime = new Date();	
	private HostInfo targetInfo = null;

	public ServerSender(HostInfo clientInfo, HostInfo targetInfo)
	{
		this.clientInfo = clientInfo;
		this.targetInfo = targetInfo;
	}
	
	public HostInfo getClientInfo()
	{
		return clientInfo;
	}
	
	/**
	 * Connection is considered dead if the lastReceivedPacketTime Date is older
	 * than MAX_LAST_RECEIVED_TIME_MS.
	 * @return
	 */
	public boolean isConnectionDead()
	{
		long lastReceivedMs = lastReceivedPacketTime.getTime();
		long currentMs = new Date().getTime();
		
		return (currentMs - lastReceivedMs) > MAX_LAST_RECEIVED_TIME_MS;
	}
	
	public boolean isClientInfoEqual(HostInfo clientInfo)
	{
		return this.clientInfo.equals(clientInfo);
	}

	public void quit()
	{
		bKeepRunning = false;
	}

	@Override
	public void run()
	{
		try
		{
			threadMain();
		}
		catch (Exception exc)
		{
			execThreadFatalErrorListeners(exc);
		}

		cleanUpSocket();
	}

	private void threadMain() throws Exception
	{
		socket = new DatagramSocket();
		socket.setSoTimeout(1);

		while (bKeepRunning)
		{
			sendAllPackets(targetInfo);

			DatagramPacket packet = new DatagramPacket(new byte[PACKET_SIZE], PACKET_SIZE);

			try
			{
				socket.receive(packet);
				
				lastReceivedPacketTime = new Date();
				execPacketReceivedListeners(packet);
			}
			catch (SocketTimeoutException exc)
			{
				// Not an error.
			}
		}
	}
}
