package zalewa.delayedproxy.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.BindException;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.text.DecimalFormat;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import zalewa.delayedproxy.VersionInfo;
import zalewa.delayedproxy.exceptions.IncorrectPortNumberException;
import zalewa.delayedproxy.exceptions.LoopbackException;
import zalewa.delayedproxy.exceptions.ProxyStartException;
import zalewa.delayedproxy.network.DatagramBase;
import zalewa.delayedproxy.network.HostInfo;
import zalewa.delayedproxy.network.ProxyManager;
import zalewa.delayedproxy.network.eventlisteners.ThreadFatalErrorListener;

public class MainWindow extends JFrame
{
	class MyData
	{
		public boolean bIsProxyRunning = false;
		public ProxyManager proxy = new ProxyManager();
	}

	class MyThreadFatalErrorListener implements Runnable,
			ThreadFatalErrorListener
	{
		Exception error = null;

		@Override
		public synchronized void fatalError(DatagramBase datagramBase, Exception error)
		{
			MyThreadFatalErrorListener dispatcher = new MyThreadFatalErrorListener();
			dispatcher.error = error;

			SwingUtilities.invokeLater(dispatcher);
		}

		@Override
		public void run()
		{
			if (error instanceof BindException)
			{
				displayProxyThreadError("Local Port is already bound!\n"
						+ "Please select different Local Port.");
			}
			else
			{
				displayProxyThreadError(error);
			}

			stopProxy();
			lblStatusBar.setText("Proxy stopped due to an ERROR!");
		}
	}

	private static final long serialVersionUID = 1L;
	private static final int statisticsUpdateDelayMs = 1000;

	private JButton btnToggleProxy = null;

	private final MyData data = new MyData();
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JLabel jLabel3 = null;
	private JLabel jLabel4 = null;
	private JLabel jLabel5 = null;
	private JLabel jLabel6 = null;
	private JPanel jPanel = null;
	private JPanel jPanel1 = null;
	private JPanel jPanel2 = null;

	private JPanel jPanel3 = null;

	private JTextArea jTextArea = null;

	private JLabel lblStatusBar = null;

	private JSlider sliderPacketLoss = null;

	private JSlider sliderPing = null;

	private JSpinner spinLocalPort = null;

	private JSpinner spinPacketLoss = null;

	private JSpinner spinPing = null;

	private Timer statisticsUpdateTimer = null;

	private JTextField tePacketsLostToClient = null;

	private JTextField tePacketsLostToClientPercentage = null;

	private JTextField tePacketsLostToServer = null;

	private JTextField tePacketsLostToServerPercentage = null;

	private JTextField tePacketsSentToClient = null;

	private JTextField tePacketsSentToServer = null;

	private JTextField teTargetAddress = null;

	/**
	 * This is the default constructor
	 */
	public MainWindow()
	{
		super();
		initialize();
	}

	/**
	 * This method initializes btnToggleProxy
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getBtnToggleProxy()
	{
		if (btnToggleProxy == null)
		{
			btnToggleProxy = new JButton();
			btnToggleProxy.setText("START");
			btnToggleProxy.addActionListener(new java.awt.event.ActionListener()
			{
				public void actionPerformed(java.awt.event.ActionEvent e)
				{
					toggleProxy();
				}
			});
		}
		return btnToggleProxy;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane()
	{
		if (jContentPane == null)
		{
			lblStatusBar = new JLabel();
			lblStatusBar.setBounds(new Rectangle(15, 435, 466, 16));
			lblStatusBar.setFont(new Font("Dialog", Font.PLAIN, 12));
			lblStatusBar.setHorizontalTextPosition(SwingConstants.CENTER);
			lblStatusBar.setHorizontalAlignment(SwingConstants.CENTER);
			lblStatusBar.setText("<Status Bar>");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getJPanel(), null);
			jContentPane.add(getJPanel1(), null);
			jContentPane.add(getJPanel2(), null);
			jContentPane.add(lblStatusBar, null);
			jContentPane.add(getJPanel3(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel()
	{
		if (jPanel == null)
		{
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 2;
			gridBagConstraints4.gridheight = 2;
			gridBagConstraints4.insets = new Insets(0, 5, 0, 5);
			gridBagConstraints4.ipadx = 0;
			gridBagConstraints4.fill = GridBagConstraints.NONE;
			gridBagConstraints4.gridy = 1;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints1.gridy = 2;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints1.gridx = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagConstraints.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints.gridy = 2;
			jLabel = new JLabel();
			jLabel.setText("Local port:");
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints3.gridy = 1;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints3.gridx = 1;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.anchor = GridBagConstraints.EAST;
			gridBagConstraints2.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints2.gridy = 1;
			jLabel1 = new JLabel();
			jLabel1.setText("Target server address:");
			jPanel = new JPanel();
			jPanel.setBorder(BorderFactory.createTitledBorder(null, "Proxy",
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION,
					new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51)));
			jPanel.setLayout(new GridBagLayout());
			jPanel.setBounds(new Rectangle(90, 15, 391, 76));
			jPanel.add(jLabel1, gridBagConstraints2);
			jPanel.add(getTeTargetAddress(), gridBagConstraints3);
			jPanel.add(jLabel, gridBagConstraints);
			jPanel.add(getSpinLocalPort(), gridBagConstraints1);
			jPanel.add(getBtnToggleProxy(), gridBagConstraints4);
		}
		return jPanel;
	}

	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel1()
	{
		if (jPanel1 == null)
		{
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints6.gridy = 0;
			gridBagConstraints6.weightx = 100.0;
			gridBagConstraints6.ipadx = 0;
			gridBagConstraints6.gridx = 0;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.fill = GridBagConstraints.NONE;
			gridBagConstraints5.gridy = 0;
			gridBagConstraints5.weightx = 5.0;
			gridBagConstraints5.ipadx = 0;
			gridBagConstraints5.insets = new Insets(0, 0, 0, 0);
			gridBagConstraints5.gridx = 1;
			jPanel1 = new JPanel();
			jPanel1.setLayout(new GridBagLayout());
			jPanel1.setBorder(BorderFactory.createTitledBorder(null,
					"Ping (ms)", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION,
					new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51)));
			jPanel1.setBounds(new Rectangle(15, 97, 466, 69));
			jPanel1.add(getSpinPing(), gridBagConstraints5);
			jPanel1.add(getSliderPing(), gridBagConstraints6);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jPanel2
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel2()
	{
		if (jPanel2 == null)
		{
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.fill = GridBagConstraints.NONE;
			gridBagConstraints8.gridy = 0;
			gridBagConstraints8.weightx = 5.0;
			gridBagConstraints8.ipadx = 22;
			gridBagConstraints8.gridx = 1;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints7.gridy = 0;
			gridBagConstraints7.weightx = 100.0;
			gridBagConstraints7.gridx = 0;
			jPanel2 = new JPanel();
			jPanel2.setLayout(new GridBagLayout());
			jPanel2.setBorder(BorderFactory.createTitledBorder(null,
					"Packet loss (%)", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION,
					new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51)));
			jPanel2.setBounds(new Rectangle(15, 180, 466, 61));
			jPanel2.add(getSliderPacketLoss(), gridBagConstraints7);
			jPanel2.add(getSpinPacketLoss(), gridBagConstraints8);
		}
		return jPanel2;
	}

	/**
	 * This method initializes jPanel3
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel3()
	{
		if (jPanel3 == null)
		{
			GridBagConstraints gridBagConstraints20 = new GridBagConstraints();
			gridBagConstraints20.fill = GridBagConstraints.BOTH;
			gridBagConstraints20.gridy = 0;
			gridBagConstraints20.weightx = 1.0;
			gridBagConstraints20.weighty = 100.0;
			gridBagConstraints20.gridwidth = 3;
			gridBagConstraints20.ipadx = 0;
			gridBagConstraints20.ipady = 0;
			gridBagConstraints20.insets = new Insets(0, 0, 0, 0);
			gridBagConstraints20.gridx = 0;
			GridBagConstraints gridBagConstraints18 = new GridBagConstraints();
			gridBagConstraints18.fill = GridBagConstraints.BOTH;
			gridBagConstraints18.gridy = 6;
			gridBagConstraints18.weightx = 1.0;
			gridBagConstraints18.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints18.gridx = 2;
			GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
			gridBagConstraints17.fill = GridBagConstraints.BOTH;
			gridBagConstraints17.gridy = 4;
			gridBagConstraints17.weightx = 1.0;
			gridBagConstraints17.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints17.gridx = 2;
			GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
			gridBagConstraints16.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints16.gridy = 6;
			gridBagConstraints16.weightx = 1.0;
			gridBagConstraints16.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints16.gridx = 1;
			GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
			gridBagConstraints15.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints15.gridy = 5;
			gridBagConstraints15.weightx = 1.0;
			gridBagConstraints15.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints15.gridwidth = 2;
			gridBagConstraints15.gridx = 1;
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
			gridBagConstraints14.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints14.gridy = 4;
			gridBagConstraints14.weightx = 1.0;
			gridBagConstraints14.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints14.gridx = 1;
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints13.gridy = 3;
			gridBagConstraints13.weightx = 1.0;
			gridBagConstraints13.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints13.gridwidth = 2;
			gridBagConstraints13.gridx = 1;
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.gridx = 0;
			gridBagConstraints12.anchor = GridBagConstraints.EAST;
			gridBagConstraints12.ipadx = 0;
			gridBagConstraints12.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints12.gridy = 6;
			jLabel6 = new JLabel();
			jLabel6.setText("Packets lost to Client:");
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.gridx = 0;
			gridBagConstraints10.anchor = GridBagConstraints.EAST;
			gridBagConstraints10.ipadx = 0;
			gridBagConstraints10.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints10.gridy = 4;
			jLabel5 = new JLabel();
			jLabel5.setText("Packets lost to target Server:");
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.anchor = GridBagConstraints.EAST;
			gridBagConstraints11.ipadx = 0;
			gridBagConstraints11.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints11.gridy = 5;
			jLabel4 = new JLabel();
			jLabel4.setText("Packets sent to Client:");
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.gridx = 0;
			gridBagConstraints9.anchor = GridBagConstraints.EAST;
			gridBagConstraints9.ipadx = 0;
			gridBagConstraints9.insets = new Insets(3, 3, 3, 3);
			gridBagConstraints9.gridy = 3;
			jLabel3 = new JLabel();
			jLabel3.setText("Packets sent to target Server:");
			jPanel3 = new JPanel();
			jPanel3.setLayout(new GridBagLayout());
			jPanel3.setBounds(new Rectangle(15, 255, 466, 166));
			jPanel3.setBorder(BorderFactory.createTitledBorder(null,
					"Statistics", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION,
					new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51)));
			jPanel3.add(jLabel3, gridBagConstraints9);
			jPanel3.add(jLabel4, gridBagConstraints11);
			jPanel3.add(jLabel5, gridBagConstraints10);
			jPanel3.add(jLabel6, gridBagConstraints12);
			jPanel3.add(getTePacketsSentToServer(), gridBagConstraints13);
			jPanel3.add(getTePacketsLostToServer(), gridBagConstraints14);
			jPanel3.add(getTePacketsSentToClient(), gridBagConstraints15);
			jPanel3.add(getTePacketsLostToClient(), gridBagConstraints16);
			jPanel3.add(getTePacketsLostToServerPercentage(),
					gridBagConstraints17);
			jPanel3.add(getTePacketsLostToClientPercentage(),
					gridBagConstraints18);
			jPanel3.add(getJTextArea(), gridBagConstraints20);
		}
		return jPanel3;
	}

	/**
	 * This method initializes jTextArea
	 * 
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getJTextArea()
	{
		if (jTextArea == null)
		{
			jTextArea = new JTextArea();
			jTextArea.setText("\"Lost\" packets only show number of packets deliberately lost by the application.\nPackets lost by the bad connection quality are displayed as \"sent\".");
			jTextArea.setEditable(false);
			jTextArea.setFont(new Font("Dialog", Font.ITALIC, 12));
			jTextArea.setLineWrap(true);
		}
		return jTextArea;
	}

	/**
	 * This method initializes sliderPacketLoss
	 * 
	 * @return javax.swing.JSlider
	 */
	private JSlider getSliderPacketLoss()
	{
		if (sliderPacketLoss == null)
		{
			sliderPacketLoss = new JSlider();
			sliderPacketLoss.setValue(0);
			sliderPacketLoss.setPaintTicks(true);
			sliderPacketLoss.setMajorTickSpacing(10);
			sliderPacketLoss.setMinorTickSpacing(1);
			sliderPacketLoss.setSnapToTicks(true);
			sliderPacketLoss.addChangeListener(new javax.swing.event.ChangeListener()
			{
				public void stateChanged(javax.swing.event.ChangeEvent e)
				{
					if (sliderPacketLoss.isFocusOwner())
					{
						int value = sliderPacketLoss.getValue();
						spinPacketLoss.setValue(value);

						updateProxyParameters();
					}
				}
			});
		}
		return sliderPacketLoss;
	}

	/**
	 * This method initializes sliderPing
	 * 
	 * @return javax.swing.JSlider
	 */
	private JSlider getSliderPing()
	{
		if (sliderPing == null)
		{
			sliderPing = new JSlider();
			sliderPing.setMaximum(2000);
			sliderPing.setSnapToTicks(true);
			sliderPing.setPaintTicks(true);
			sliderPing.setMajorTickSpacing(200);
			sliderPing.setMinorTickSpacing(1);
			sliderPing.setValue(0);
			sliderPing.addChangeListener(new javax.swing.event.ChangeListener()
			{
				public void stateChanged(javax.swing.event.ChangeEvent e)
				{
					if (sliderPing.isFocusOwner())
					{
						int value = sliderPing.getValue();
						spinPing.setValue(value);

						updateProxyParameters();
					}
				}
			});
		}
		return sliderPing;
	}

	/**
	 * This method initializes spinLocalPort
	 * 
	 * @return javax.swing.JTextField
	 */
	private JSpinner getSpinLocalPort()
	{
		if (spinLocalPort == null)
		{
			SpinnerNumberModel spinnerModel = new SpinnerNumberModel();

			spinnerModel.setMinimum(1);
			spinnerModel.setMaximum(65535);

			spinLocalPort = new JSpinner();
			spinLocalPort.setFont(new Font("Dialog", Font.PLAIN, 12));
			spinLocalPort.setModel(spinnerModel);
			spinLocalPort.setValue(10667);
		}
		return spinLocalPort;
	}

	/**
	 * This method initializes spinPacketLoss
	 * 
	 * @return javax.swing.JTextField
	 */
	private JSpinner getSpinPacketLoss()
	{
		if (spinPacketLoss == null)
		{
			SpinnerNumberModel spinnerModel = new SpinnerNumberModel();

			spinnerModel.setMinimum(0);
			spinnerModel.setMaximum(100);

			spinPacketLoss = new JSpinner();
			spinPacketLoss.setFont(new Font("Dialog", Font.PLAIN, 12));
			spinPacketLoss.setModel(spinnerModel);
			spinPacketLoss.setValue(0);
			spinPacketLoss.addChangeListener(new ChangeListener()
			{

				@Override
				public void stateChanged(ChangeEvent arg0)
				{
					if (UIUtils.isJSpinnerFocusOwner(spinPacketLoss))
					{
						int value = (Integer) spinPacketLoss.getValue();
						sliderPacketLoss.setValue(value);

						updateProxyParameters();
					}

				}

			});
		}
		return spinPacketLoss;
	}

	/**
	 * This method initializes spinPing
	 * 
	 * @return javax.swing.JTextField
	 */
	private JSpinner getSpinPing()
	{
		if (spinPing == null)
		{
			SpinnerNumberModel spinnerModel = new SpinnerNumberModel();

			spinnerModel.setMinimum(0);
			spinnerModel.setMaximum(2000);

			spinPing = new JSpinner();
			spinPing.setFont(new Font("Dialog", Font.PLAIN, 12));
			spinPing.setModel(spinnerModel);
			spinPing.setValue(0);
			spinPing.addChangeListener(new ChangeListener()
			{

				@Override
				public void stateChanged(ChangeEvent arg0)
				{
					if (UIUtils.isJSpinnerFocusOwner(spinPing))
					{
						int value = (Integer) spinPing.getValue();
						sliderPing.setValue(value);

						updateProxyParameters();
					}

				}

			});

		}
		return spinPing;
	}

	/**
	 * This method initializes tePacketsLostToClient
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTePacketsLostToClient()
	{
		if (tePacketsLostToClient == null)
		{
			tePacketsLostToClient = new JTextField();
			tePacketsLostToClient.setEditable(false);
			tePacketsLostToClient.setHorizontalAlignment(JTextField.TRAILING);
		}
		return tePacketsLostToClient;
	}

	/**
	 * This method initializes tePacketsLostToClientPercentage
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTePacketsLostToClientPercentage()
	{
		if (tePacketsLostToClientPercentage == null)
		{
			tePacketsLostToClientPercentage = new JTextField();
			tePacketsLostToClientPercentage.setEditable(false);
			tePacketsLostToClientPercentage.setHorizontalAlignment(JTextField.TRAILING);
		}
		return tePacketsLostToClientPercentage;
	}

	/**
	 * This method initializes tePacketsLostToServer
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTePacketsLostToServer()
	{
		if (tePacketsLostToServer == null)
		{
			tePacketsLostToServer = new JTextField();
			tePacketsLostToServer.setEditable(false);
			tePacketsLostToServer.setHorizontalAlignment(JTextField.TRAILING);
		}
		return tePacketsLostToServer;
	}

	/**
	 * This method initializes tePacketsLostToServerPercentage
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTePacketsLostToServerPercentage()
	{
		if (tePacketsLostToServerPercentage == null)
		{
			tePacketsLostToServerPercentage = new JTextField();
			tePacketsLostToServerPercentage.setEditable(false);
			tePacketsLostToServerPercentage.setHorizontalAlignment(JTextField.TRAILING);
		}
		return tePacketsLostToServerPercentage;
	}

	/**
	 * This method initializes tePacketsSentToClient
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTePacketsSentToClient()
	{
		if (tePacketsSentToClient == null)
		{
			tePacketsSentToClient = new JTextField();
			tePacketsSentToClient.setEditable(false);
			tePacketsSentToClient.setHorizontalAlignment(JTextField.TRAILING);
		}
		return tePacketsSentToClient;
	}
	
	/**
	 * This method initializes tePacketsSentToServer
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTePacketsSentToServer()
	{
		if (tePacketsSentToServer == null)
		{
			tePacketsSentToServer = new JTextField();
			tePacketsSentToServer.setEditable(false);
			tePacketsSentToServer.setHorizontalAlignment(JTextField.TRAILING);
		}
		return tePacketsSentToServer;
	}	

	/**
	 * This method initializes teTargetAddress
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTeTargetAddress()
	{
		if (teTargetAddress == null)
		{
			teTargetAddress = new JTextField();
			teTargetAddress.setText("localhost:10666");
			teTargetAddress.setHorizontalAlignment(JTextField.TRAILING);
		}
		return teTargetAddress;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize()
	{
		this.setSize(507, 499);
		this.setResizable(false);
		this.setContentPane(getJContentPane());
		this.setTitle("Delayed UDP Proxy");
	}
	
////////////////////////////////////////////////////////////////////////////////
	// End of auto generated methods.
	
	private void centerWindow()
	{
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolkit.getScreenSize();
		
		int posX = (screenSize.width - this.getWidth()) / 2;
		int posY = (screenSize.height - this.getHeight()) / 2;
		
		this.setLocation(posX, posY);
	}

	private void displayProxyThreadError(Exception exc)
	{
		displayProxyThreadError("Proxy has been stopped because following error has occured:\n"
				+ exc.getMessage());
	}

	private void displayProxyThreadError(String error)
	{
		JOptionPane.showMessageDialog(this, error, "Proxy Thread Error",
				JOptionPane.ERROR_MESSAGE);
	}

	private void setupCustoms()
	{
		centerWindow();

		lblStatusBar.setText(VersionInfo.getCopyrightInfo());
		data.proxy.setThreadFatalErrorListener(new MyThreadFatalErrorListener());

		setupStatisticsTimer();
		setupThisWindowListener();
	}

	private void setupStatisticsTimer()
	{
		statisticsUpdateTimer = new Timer(statisticsUpdateDelayMs, new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				updateStatistics();
			}
		});

		statisticsUpdateTimer.setDelay(statisticsUpdateDelayMs);
		statisticsUpdateTimer.setInitialDelay(statisticsUpdateDelayMs);
	}

	private void setupThisWindowListener()
	{
		this.addWindowListener(new WindowAdapter()
		{
			@Override
			public void windowClosing(WindowEvent e)
			{
				stopProxy();
				System.exit(0);
			}
		});
	}

	private void startProxy() throws ProxyStartException
	{
		String targetAddress = teTargetAddress.getText();
		int localPort = (Integer) spinLocalPort.getValue();

		int pingMs = sliderPing.getValue();
		int packetLossPercentage = sliderPacketLoss.getValue();

		lblStatusBar.setText("Starting proxy");

		try
		{
			localPort = (Integer) spinLocalPort.getValue();

			HostInfo.validatePort(localPort);
		}
		catch (Exception exc)
		{
			JOptionPane.showMessageDialog(this,
					"Port not specified or invalid (must be 1 - 65535).",
					"Error", JOptionPane.ERROR_MESSAGE);

			throw new ProxyStartException();
		}

		try
		{
			data.proxy.setPacketLossPercentage(packetLossPercentage);
			data.proxy.setPacketDelay(pingMs);
			data.proxy.startThreads(targetAddress, localPort);
		}
		catch (LoopbackException exc)
		{
			JOptionPane.showMessageDialog(
					this,
					"Local and remote ports are the same.\n"
							+ "This will result in an infinite loop. ABORTING.",
					"Error", JOptionPane.ERROR_MESSAGE);

			throw new ProxyStartException();
		}
		catch (IncorrectPortNumberException exc)
		{
			JOptionPane.showMessageDialog(this,
					"Port invalid (must be 1 - 65535).", "Error",
					JOptionPane.ERROR_MESSAGE);

			throw new ProxyStartException();
		}
		catch (MalformedURLException exc)
		{
			JOptionPane.showMessageDialog(this, "Incorrect server address.",
					"Error", JOptionPane.ERROR_MESSAGE);

			throw new ProxyStartException();
		}
		catch (UnknownHostException exc)
		{
			JOptionPane.showMessageDialog(this, "Unknown host: "
					+ targetAddress, "Error", JOptionPane.ERROR_MESSAGE);

			throw new ProxyStartException();
		}

		if (data.proxy.isProxyRunning())
		{
			statisticsUpdateTimer.start();
			btnToggleProxy.setText("STOP");
			data.bIsProxyRunning = true;

			lblStatusBar.setText("Proxy is now running!");
		}
	}

	private void stopProxy()
	{
		data.proxy.stopThreads();

		statisticsUpdateTimer.stop();
		btnToggleProxy.setText("START");
		data.bIsProxyRunning = false;

		lblStatusBar.setText("Proxy stopped.");
	}

	private void toggleProxy()
	{
		if (data.bIsProxyRunning)
		{
			stopProxy();
		}
		else
		{
			try
			{
				startProxy();
			}
			catch (ProxyStartException exc)
			{
				stopProxy();
				lblStatusBar.setText("FAILED to start the proxy.");
			}
		}
	}

	private void updateProxyParameters()
	{
		if (data.proxy != null)
		{
			int delayMs = sliderPing.getValue();
			int packetLossChance = sliderPacketLoss.getValue();

			data.proxy.setPacketDelay(delayMs);
			data.proxy.setPacketLossPercentage(packetLossChance);
		}
	}

	private void updateStatistics()
	{
		int packetsLostToClient = data.proxy.getNumPacketsLostToClient();
		int packetsLostToServer = data.proxy.getNumPacketsLostToServer();
		int packetsSentToClient = data.proxy.getNumPacketsSentToClient();
		int packetsSentToServer = data.proxy.getNumPacketsSentToServer();

		int packetsClientTotal = packetsLostToClient + packetsSentToClient;
		int packetsServerTotal = packetsLostToServer + packetsSentToServer;

		tePacketsLostToClient.setText(Integer.toString(packetsLostToClient));
		tePacketsLostToServer.setText(Integer.toString(packetsLostToServer));
		tePacketsSentToClient.setText(Integer.toString(packetsSentToClient));
		tePacketsSentToServer.setText(Integer.toString(packetsSentToServer));

		updateStatisticsSetPercentage(packetsLostToClient, packetsClientTotal,
				tePacketsLostToClientPercentage);
		updateStatisticsSetPercentage(packetsLostToServer, packetsServerTotal,
				tePacketsLostToServerPercentage);
	}

	private void updateStatisticsSetPercentage(int numerator, int denumerator, JTextField textBox)
	{
		if (denumerator != 0)
		{
			float percentage = (float) numerator / (float) denumerator;
			textBox.setText(DecimalFormat.getPercentInstance().format(
					percentage));
		}
		else
		{
			textBox.setText("N/A");
		}
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				MainWindow thisClass = new MainWindow();
				thisClass.setupCustoms();
				thisClass.setVisible(true);

			}
		});
	}	

} // @jve:decl-index=0:visual-constraint="27,8"
